#include "arduinoFFT.h"

#define SAMPLES 64 // SAMPLES-pt FFT. Must be a base 2 number. Max 128 for Arduino Uno.
#define SAMPLING_FREQUENCY 2048 // Ts = Based on Nyquist, must be 2 times the highest expected frequency.
#define COLLECTION_TIME 500 // Time to collect samples in milliseconds (1 second)
#define ACTION_TIME 500 // Time to act
#define LOUDNESS_THRESHOLD 100 // The loudness threshold for the specific frequency

arduinoFFT FFT = arduinoFFT();

unsigned long endTime;
unsigned long actionTime;
unsigned long microSeconds;

double R_vReal[SAMPLES]; // Create vector of size SAMPLES to hold real values
double R_vImag[SAMPLES]; // Create vector of size SAMPLES to hold imaginary values

double L_vReal[SAMPLES]; // Create vector of size SAMPLES to hold real values
double L_vImag[SAMPLES]; // Create vector of size SAMPLES to hold imaginary values

int motor1pin1 = 2;
int motor1pin2 = 3;

int motor2pin1 = 4;
int motor2pin2 = 5;

void setup()
{
  Serial.begin(115200); // Baud rate for the Serial Monitor

  pinMode(13,OUTPUT);

  pinMode(motor1pin1, OUTPUT);
  pinMode(motor1pin2, OUTPUT);
  pinMode(motor2pin1, OUTPUT);
  pinMode(motor2pin2, OUTPUT);

  pinMode(9, OUTPUT); 
  pinMode(10, OUTPUT);

  digitalWrite(motor1pin1, HIGH);
  digitalWrite(motor1pin2, LOW);

  digitalWrite(motor2pin1, LOW);
  digitalWrite(motor2pin2, HIGH);
}

void loop()
{ 
  digitalWrite(13,HIGH);
  
  // Variables to hold the accumulated loudness for each frequency range for Right Ear
  double R_loudness_LF = 0;
  double R_loudness_MF1 = 0;
  double R_loudness_HF1 = 0;
  double R_loudness_HF2 = 0;

 // Variables to hold the accumulated loudness for each frequency range for Left Ear
  double L_loudness_LF = 0;
  double L_loudness_MF1 = 0;
  double L_loudness_HF1 = 0;
  double L_loudness_HF2 = 0;

  double R_maxLoudness = 0; // Variable to hold the maximum loudness
  double R_maxFrequency = 0; // Variable to hold the frequency of the maximum loudness

  double L_maxLoudness = 0; // Variable to hold the maximum loudness
  double L_maxFrequency = 0; // Variable to hold the frequency of the maximum loudness

  // Record the end time for sample collection
  endTime = millis() + COLLECTION_TIME;

  // Sample and accumulate loudness for 1 second
  while (millis() < endTime)
  {
    for (int i = 0; i < SAMPLES; i++)
    {
      microSeconds = micros(); // Returns the number of microseconds since the Arduino board began running the current script.

      R_vReal[i] = analogRead(5); // Reads the value from analog pin 0 (A0), quantize it and save it as a real term.
      R_vImag[i] = 0;             // Makes imaginary term 0 always

      L_vReal[i] = analogRead(4); // Reads the value from analog pin 0 (A0), quantize it and save it as a real term.
      L_vImag[i] = 0;             // Makes imaginary term 0 always

      // Remaining wait time between samples if necessary
      while (micros() < (microSeconds + (1000000.0 / SAMPLING_FREQUENCY)))
      {
        // Do nothing
      }
    }

    // Perform FFT on samples for Right Ear
    FFT.Windowing(R_vReal, SAMPLES, FFT_WIN_TYP_HAMMING, FFT_FORWARD);
    FFT.Compute(R_vReal, R_vImag, SAMPLES, FFT_FORWARD);
    FFT.ComplexToMagnitude(R_vReal, R_vImag, SAMPLES);

    // Accumulate loudness for each frequency range and find the loudest frequency
    for (int i = 0; i < SAMPLES / 2; i++)
    {
      double frequency = ((double)i * SAMPLING_FREQUENCY) / SAMPLES; // Calculate frequency in Hz

      if (frequency >= 210 && frequency < 300)
      {
        R_loudness_LF += R_vReal[i];
      }
      else if (frequency >= 300 && frequency < 380)
      {
        R_loudness_MF1 += R_vReal[i];
      }
      else if (frequency >= 550 && frequency < 630)
      {
        R_loudness_HF1 += R_vReal[i];
      }
      else if (frequency >= 630 && frequency < 740)
      {
        R_loudness_HF2 += R_vReal[i];
      }

      // Check if this is the loudest frequency so far, only considering frequencies > 200 Hz
      if (frequency > 200 && R_vReal[i] > R_maxLoudness)
      {
        R_maxLoudness = R_vReal[i];
        R_maxFrequency = frequency;
      }
    }

      // Perform FFT on samples for Left Ear
    FFT.Windowing(L_vReal, SAMPLES, FFT_WIN_TYP_HAMMING, FFT_FORWARD);
    FFT.Compute(L_vReal, L_vImag, SAMPLES, FFT_FORWARD);
    FFT.ComplexToMagnitude(L_vReal, L_vImag, SAMPLES);

    // Accumulate loudness for each frequency range and find the loudest frequency
    for (int i = 0; i < SAMPLES / 2; i++)
    {
      double frequency = ((double)i * SAMPLING_FREQUENCY) / SAMPLES; // Calculate frequency in Hz

      if (frequency >= 210 && frequency < 300)
      {
        L_loudness_LF += L_vReal[i];
      }
      else if (frequency >= 300 && frequency < 380)
      {
        L_loudness_MF1 += L_vReal[i];
      }
      else if (frequency >= 550 && frequency < 630)
      {
        L_loudness_HF1 += L_vReal[i];
      }
      else if (frequency >= 630 && frequency < 740)
      {
        L_loudness_HF2 += L_vReal[i];
      }

      // Check if this is the loudest frequency so far, only considering frequencies > 200 Hz
      if (frequency > 200 && L_vReal[i] > L_maxLoudness)
      {
        L_maxLoudness = L_vReal[i];
        L_maxFrequency = frequency;
      }
    }
  }

  // Print the accumulated loudness for each frequency range
  Serial.println("Accumulated Loudness for Each Frequency Range:");
  Serial.print("210-300 Hz: ");
  Serial.print(L_loudness_LF);
  Serial.print("------");
  Serial.println(R_loudness_LF);
  // Serial.println("\n --- (Move Forward)");

  Serial.print("300-380 Hz: ");
  Serial.print(L_loudness_MF1);
  Serial.print("------");
  Serial.println(R_loudness_MF1);
  // Serial.println(" --- (Move Backward)");

  Serial.print("550-630 Hz: ");
  Serial.print(L_loudness_HF1);
  Serial.print("------");
  Serial.println(R_loudness_HF1);
  // Serial.println(" --- (Rotate Right)");

  Serial.print("630-740 Hz: ");
  Serial.print(L_loudness_HF2);
  Serial.print("------");
  Serial.println(R_loudness_HF2);
  // Serial.println(" --- (Rotate Left)");

  // // Print the loudest frequency and its loudness
  // Serial.print("Loudest Frequency (>200 Hz): ");
  // Serial.print(R_maxFrequency);
  // Serial.print(" Hz, Loudness: ");
  // Serial.print(R_maxLoudness);

  // // Check if the loudness of the loudest frequency exceeds the threshold
  // if (R_maxLoudness > LOUDNESS_THRESHOLD)
  // {
  //   Serial.println("Specific action for loudest frequency loudness exceeding 100");
  // }

  // Serial.println();

  // Determine the action based on the range with the highest accumulated loudness

  if (abs(R_loudness_HF2 + L_loudness_HF2) < 200)
  {
    Serial.println("I HEAR NOTHING");
    digitalWrite(motor2pin1, LOW);
    digitalWrite(motor2pin2, HIGH);
    analogWrite(9, 0); //ENA   pin
    analogWrite(10, 0); //ENB pin
    actionTime = 0;
  }

  else if (abs(R_loudness_HF2- L_loudness_HF2) < 100)
  {
    Serial.println("LOOKING GOOD");
    digitalWrite(motor2pin1, LOW);
    digitalWrite(motor2pin2, HIGH);
    analogWrite(9, 160); //ENA   pin
    analogWrite(10, 180); //ENB pin
    actionTime = 1300;
  }

  else if (R_loudness_HF2 > L_loudness_HF2)
  {
    Serial.println("TURN RIGHT");
    digitalWrite(motor2pin1, LOW);
    digitalWrite(motor2pin2, HIGH);
    analogWrite(9, 100); //ENA   pin
    analogWrite(10, 180); //ENB pin
    actionTime = 400;
  }

  else if (R_loudness_HF2 < L_loudness_HF2)
  {
    Serial.println("TURN LEFT");
    digitalWrite(motor2pin1, LOW);
    digitalWrite(motor2pin2, HIGH);
    analogWrite(9, 180); //ENA   pin
    analogWrite(10, 100); //ENB pin
    actionTime = 400;
  }
  // else if (R_loudness_MF1 > R_loudness_LF && R_loudness_MF1 > R_loudness_HF1 && R_loudness_MF1 > R_loudness_HF2)
  // {
  //   Serial.println("Move Backward");
  //   digitalWrite(motor2pin1, HIGH);
  //   digitalWrite(motor2pin2, LOW);
  //   analogWrite(9, 200); //ENA   pin
  //   analogWrite(10, 200); //ENB pin
  // }
  // else if (loudness_HF1 > R_loudness_LF && R_loudness_HF1 > R_loudness_MF1 && R_loudness_HF1 > R_loudness_HF2)
  // {
  //   Serial.println("Rotate Right");
  //   digitalWrite(motor2pin1, LOW);
  //   digitalWrite(motor2pin2, HIGH);
  //   analogWrite(9, 200); //ENA   pin
  //   analogWrite(10, 80); //ENB pin
  // }
  // else if (loudness_HF2 > loudness_LF && loudness_HF2 > loudness_MF1 && loudness_HF2 > loudness_HF1)
  // {
  //   Serial.println("Rotate Left");
  //   digitalWrite(motor2pin1, LOW);
  //   digitalWrite(motor2pin2, HIGH);
  //   analogWrite(9, 80); //ENA   pin
  //   analogWrite(10, 200); //ENB pin
  // }
  // else
  // {
  //   Serial.println("No action detected");
  // }

  Serial.println("---------------------------------------");
  
  // Delay before the next sample collection
  delay(actionTime); // Adjust as needed for your application
  digitalWrite(motor2pin1, LOW);
  digitalWrite(motor2pin2, HIGH);
  analogWrite(9, 0); //ENA   pin
  analogWrite(10, 0); //ENB pin
  delay(200);
}
